# DestinE Analytics

A comprehensive analytics toolkit for data analysis, visualization, and log processing, specifically designed for AutoSubmit (AS) workflow analysis and file system exploration.

## Overview

DestinE Analytics provides powerful tools for:
- **File System Analysis**: Generate and visualize file trees with size analysis
- **Log Processing**: Parse and analyze AutoSubmit logs for workflow monitoring
- **Gantt Chart Generation**: Create detailed timeline visualizations of job execution
- **Data Visualization**: Interactive plots using both Matplotlib and Plotly

## Features

### 🗂️ File Tree Analysis
- Generate comprehensive file tree listings with metadata
- Interactive file size visualizations (treemap, sunburst, icicle charts)
- Support for remote file system analysis via SSH
- Caching system for improved performance

### 📊 AutoSubmit Log Analysis
- Parse AutoSubmit log files and extract job execution data
- Generate Gantt charts showing job timelines and dependencies
- Track job status, completion rates, and failure analysis
- Support for ensemble runs and chunk-based workflows

### 📈 Data Visualization
- Multiple chart types: treemap, sunburst, icicle, Gantt charts
- Interactive Plotly visualizations
- Matplotlib-based static plots
- Customizable styling and color schemes

### 🔧 Utility Tools
- Shell scripts for file tree generation
- Log transfer utilities for remote systems
- Caching mechanisms for performance optimization

## Installation

### Prerequisites
- Python 3.8 or higher
- Bash shell (for shell scripts)

### Setup

1. **Clone the repository**:
   ```bash
   git clone <repository-url>
   cd destine_analytics
   ```

2. **Create a virtual environment**:
   ```bash
   python -m venv .venv
   source .venv/bin/activate  # On Windows: .venv\Scripts\activate
   ```

3. **Install dependencies**:
   ```bash
   pip install -r requirements.txt
   ```

4. **Install the package**:
   ```bash
   pip install -e .
   ```

5. **Configure environment** (optional):
   ```bash
   cp .env.example .env
   # Edit .env with your configuration
   ```

## Usage

### Command Line Tools

**💡 Tip**: Use `-h` or `--help` with any command to see all available options and detailed usage information.

#### 1. File Tree Visualization (`draw-filetree`)

Generate interactive file tree visualizations from directories or pre-generated file listings.

```bash
# See all available options
draw-filetree -h

# Analyze a local directory
draw-filetree /path/to/directory --figure treemap --interactive

# Use pre-generated file tree output
draw-filetree tools/ls_filetree_Documents.txt --figure sunburst

# Remote directory analysis
draw-filetree /remote/path --ssh-command "ssh user@host" --figure icicle

# Save output
draw-filetree /path/to/directory --figure all --output figures/
```

**Options**:
- `--figure`: Chart type (`treemap`, `sunburst`, `icicle`, `all`)
- `--interactive`: Show interactive plot
- `--only-directories`: Show only directories (for large trees)
- `--root`: Specify root directory for subtree analysis
- `--cache`: Enable caching for performance
- `--output`: Output directory for saved figures

#### 2. Gantt Chart Generation (`create-ganttchart`)

Create Gantt charts from AutoSubmit logs showing job execution timelines.

**Note**: Currently, running `tools/create_filetree.sh` is a requirement for `create-ganttchart` to function properly.

```bash
# See all available options
create-ganttchart -h

# Basic Gantt chart
create-ganttchart o005 --interactive

# Filter by chunk range
create-ganttchart o005 --chunks 38 52 --ignore AQUA

# Color by different attributes
create-ganttchart o005 --color job --starttype submitted

# Add statistical information
create-ganttchart o005 --add-information stats --durations

# Save with custom settings
create-ganttchart o005 --output figures/ --dpi 300
```

**Options**:
- `--chunks`: Specify chunk range to analyze
- `--ignore`: Ignore jobs with specific prefixes
- `--color`: Color scheme (`job`, `chunk`, `status`)
- `--starttype`: Use `started` or `submitted` timestamps
- `--add-information`: Add extra info (`available`, `stats`)
- `--durations`: Show job durations
- `--pivot`: Organize by chunk in y-axis

### Shell Tools

#### 1. File Tree Generation (`tools/create_filetree.sh`)

Generate comprehensive file listings with metadata.

```bash
# Generate file tree listing
./tools/create_filetree.sh /path/to/directory

# Output will be saved as ls_filetree_<dirname>.txt
```

#### 2. Log Transfer (`tools/destine-vm-log-transfer.sh`)

Transfer logs from remote AutoSubmit systems.

```bash
# Transfer logs for experiment
./tools/destine-vm-log-transfer.sh o005 --output /local/path
```

**Requirements**: Configure `.env` file with SSH credentials and paths.

### Python API

#### File Tree Analysis

```python
from destine_analytics.lsreport import read_filetree
from destine_analytics.scripts.draw_filetree import draw

# Read file tree data
df = read_filetree("ls_filetree_output.txt")

# Create visualization
sizes = df.groupby("Parent")["Filesize"].sum().to_dict()
parents = {row["Filename"]: row["Parent"] for _, row in df.iterrows()}

draw(sizes, parents, figure="treemap", interactive=True)
```

#### Log Analysis

```python
from destine_analytics.logreport import get_aslogs_df, jobs_df_from_aslogs_df
from destine_analytics.plots import plot_job_gantt_chart

# Parse AutoSubmit logs
aslogs_df = get_aslogs_df(Path("path/to/aslogs"))
jobs_df = jobs_df_from_aslogs_df(aslogs_df, Path("path/to/aslogs"))

# Create Gantt chart
fig = plot_job_gantt_chart(
    jobs_df,
    title="Job Execution Timeline",
    color_key="job",
    pivot="on_overlap"
)
```

#### Plotting Decorators

```python
from destine_analytics.decorators.asrun import asrun_with_stats

# Add statistical information to plots
@asrun_with_stats(log_path=Path("logs"), after=datetime(2024, 1, 1))
def my_plot_function(df):
    # Your plotting code here
    pass
```

## Configuration

### Environment Variables

Create a `.env` file by copying from the provided template:

```bash
# Copy the example configuration
cp .env.example .env

# Edit .env with your specific configuration
```

The `.env` file should contain the following variables:

```bash
# Log directory path
LOG_DIR=/path/to/logs

# SSH configuration for remote access
file1=/path/to/ssh/key1
file2=/path/to/ssh/key2
user1=username1
user2=username2
machine1=machine1.example.com
machine2=machine2.example.com
logdir=/local/log/directory
```

### Cache Configuration

The package uses a caching system located at `~/.cache/` to improve performance for repeated operations. Cache files are automatically managed and can be cleared manually if needed.

## Project Structure

```
destine_analytics/
├── destine_analytics/          # Main package
│   ├── scripts/               # Command-line entry points
│   │   ├── draw_filetree.py   # File tree visualization
│   │   ├── create_ganttchart.py # Gantt chart generation
│   │   └── utils.py           # Shared utilities
│   ├── decorators/            # Plotting decorators
│   │   └── asrun.py           # AutoSubmit run decorators
│   ├── logreport.py           # Log parsing and analysis
│   ├── lsreport.py            # File tree parsing
│   ├── plots.py               # Visualization functions
│   └── paths.py               # Path configuration
├── tools/                     # Shell utilities
│   ├── create_filetree.sh     # File tree generation
│   └── destine-vm-log-transfer.sh # Log transfer utility
├── requirements.txt           # Python dependencies
├── pyproject.toml            # Package configuration
└── README.md                 # This file
```

## Dependencies

### Core Dependencies
- **matplotlib**: Static plotting and visualization
- **plotly**: Interactive visualizations
- **pandas**: Data manipulation and analysis
- **numpy**: Numerical computations
- **tqdm**: Progress bars for long operations

### Optional Dependencies
- **pytest**: Testing framework
- **black**: Code formatting
- **isort**: Import sorting
- **flake8**: Code linting

## Development

### Setting up Development Environment

```bash
# Install development dependencies
pip install -e ".[dev]"

# Run tests
pytest

# Format code
black destine_analytics/
isort destine_analytics/

# Lint code
flake8 destine_analytics/
```

### Adding New Features

1. **Scripts**: Add new command-line tools in `destine_analytics/scripts/`
2. **Core Functions**: Add utility functions in appropriate modules
3. **Visualizations**: Add new plotting functions in `plots.py`
4. **Decorators**: Add new decorators in `destine_analytics/decorators/`

### Testing

```bash
# Run all tests
pytest

# Run with coverage
pytest --cov=destine_analytics

# Run specific test file
pytest tests/test_logreport.py
```

## Examples

### Example 1: File System Analysis

```bash
# Generate file tree for a large directory
./tools/create_filetree.sh /home/user/documents

# Create interactive treemap visualization
draw-filetree tools/ls_filetree_documents.txt \
    --figure treemap \
    --interactive \
    --output figures/
```

### Example 2: Workflow Analysis

```bash
# Create Gantt chart for AutoSubmit experiment
create-ganttchart o005 \
    --chunks 1 50 \
    --color chunk \
    --add-information stats \
    --durations \
    --output figures/
```

### Example 3: Remote Analysis

```bash
# Analyze remote file system
draw-filetree /remote/path \
    --ssh-command "ssh user@remote-host" \
    --figure sunburst \
    --cache \
    --output remote_analysis/
```

## Troubleshooting

### Common Issues

1. **Permission Denied**: Ensure shell scripts are executable
   ```bash
   chmod +x tools/*.sh
   ```

2. **SSH Connection Issues**: Verify SSH configuration in `.env` file

3. **Memory Issues**: Use `--only-directories` flag for large file trees

4. **Cache Issues**: Clear cache directory if needed
   ```bash
   rm -rf ~/.cache/draw_filetree/
   ```

### Performance Tips

- Use caching for repeated operations
- Filter large datasets with `--chunks` or `--root`
- Use `--only-directories` for very large file trees
- Consider using remote analysis for large remote directories

## Contributing

1. Fork the repository
2. Create a feature branch
3. Make your changes
4. Add tests for new functionality
5. Ensure code passes linting and formatting
6. Submit a pull request

## License

[Add your license information here]

## Support

For issues and questions:
- Create an issue in the repository
- Check the troubleshooting section
- Review the examples for usage patterns

## Roadmap

- [ ] Add support for more visualization types
- [ ] Implement real-time log monitoring
- [ ] Add web-based dashboard
- [ ] Support for more log formats
- [ ] Enhanced caching and performance optimizations
