from datetime import datetime
from functools import wraps
from pathlib import Path
from typing import Optional, Sequence
import re

import matplotlib.pyplot as plt
import numpy as np


def _filter_fun(x):
    return "Number of jobs available: " in x


def _map_fun(x):
    return int(x.split("Number of jobs available: ")[1].strip())


def _get_lines_from_log_path(log_path: Path, after: Optional[str] = None):
    as_run_times = []
    nr_of_jobs_available = {}

    files = sorted(log_path.glob("*_run.log"))
    for file in files:
        with open(file, "r", encoding="utf-8") as rf:
            date = datetime.strptime(
                file.name.removesuffix("_run.log"), r"%Y%m%d_%H%M%S"
            )
            if after is not None and date < after:
                continue
            as_run_times.append(date)
            lines = rf.read().splitlines()
            relevant_lines = list(filter(_filter_fun, lines))
            nr_of_jobs_available[date] = sorted(set(map(_map_fun, relevant_lines)))

    return as_run_times, nr_of_jobs_available


def _get_stats_from_log_path(log_path: Path, after: Optional[datetime] = None):
    """Extract statistical information from AS logs."""
    stats = {}

    files = sorted(log_path.glob("*_run.log"))
    for file in files:
        date = datetime.strptime(file.name.removesuffix("_run.log"), r"%Y%m%d_%H%M%S")
        if after is not None and date < after:
            continue

        with open(file, "r", encoding="utf-8") as rf:
            content = rf.read()

            # Count failed and completed jobs
            failed = len(re.findall(r"Couldn't find .* COMPLETED file", content))
            completed = len(re.findall(r"Job .* is COMPLETED", content))

            # Count running jobs
            running = len(re.findall(r"Job .* is RUNNING", content))

            # Count transfer jobs
            transfer = len(
                re.findall(r"Section:TRANSFER.*can submit \d+ jobs", content)
            )

            stats[date] = {
                "failed": failed,
                "completed": completed,
                "running": running,
                "transfer": transfer,
                "ratio": float("inf") if completed == 0 else failed / completed,
            }

    return stats


def asrun_with_jobs_available(
    func,
    log_path: Path,
    after: Optional[datetime] = None,
    rel_y_position: float | Sequence[float] = 0.8,
    color_text: str = "#BBBBBB",
    color_line: str = "#BBBBBB",
):
    @wraps(func)
    def plotter(*args, **kwargs):
        local_rel_y_position = rel_y_position
        if isinstance(local_rel_y_position, float):
            local_rel_y_position = [local_rel_y_position]
        as_run_times, nr_of_jobs_available = _get_lines_from_log_path(log_path, after)
        fig = func(*args, **kwargs)
        ax = fig.get_axes()[0]
        y0, y1 = ax.get_ylim()
        local_rel_y_position = (y1 - y0) * np.array(local_rel_y_position) + y0
        for as_run_time in as_run_times:
            ax.axvline(as_run_time, color=color_line, ls="-", zorder=-1)
            nr_jobs = nr_of_jobs_available[as_run_time]
            text = (
                f"Num. jobs: {', '.join(str(element) for element in nr_jobs)}"
                if nr_jobs
                else ""
            )
            for yy in local_rel_y_position:
                ax.text(
                    as_run_time,
                    yy,
                    text,
                    color=color_text,
                    rotation="vertical",
                    zorder=-1,
                )
        return fig

    return plotter


def asrun_with_stats(
    func,
    log_path: Path,
    after: Optional[datetime] = None,
    rel_y_position: float | Sequence[float] = 0.9,
    color_text: str = "#006600",
):
    """Decorator that adds statistical information to the Gantt chart."""

    @wraps(func)
    def plotter(*args, **kwargs):
        local_rel_y_position = rel_y_position
        if isinstance(local_rel_y_position, float):
            local_rel_y_position = [local_rel_y_position]

        stats = _get_stats_from_log_path(log_path, after)
        fig = func(*args, **kwargs)
        ax = fig.get_axes()[0]
        y0, y1 = ax.get_ylim()
        local_rel_y_position = (y1 - y0) * np.array(local_rel_y_position) + y0

        # Get colormap for lines and labels
        cmap = plt.get_cmap("tab20")

        # Create header and text box content with monospace formatting
        header = "Run  | Failed | Completed | Running | Transfer | Ratio"
        separator = "-" * len(header)
        text_content = [header, separator]

        for i, (date, stat) in enumerate(stats.items(), 1):
            text_content.append(
                f"Run {i:2d} | {stat['failed']:6d} | {stat['completed']:9d} | "
                f"{stat['running']:7d} | {stat['transfer']:8d} | "
                f"{stat['ratio']:6.2f}"
            )

        # Add vertical lines and run labels with different colors
        for i, (date, stat) in enumerate(stats.items(), 1):
            # Get color from colormap, cycling through if more runs than colors
            color = cmap(i % 20)
            ax.axvline(date, color=color, ls="-", zorder=-1)

            # Add run label and stats at each y position
            for yy in local_rel_y_position:
                # Add run label
                ax.text(
                    date,
                    yy,
                    f"Run {i}",
                    color=color,
                    rotation="vertical",
                    zorder=-1,
                )

        # Add text box with monospace font
        if text_content:
            text_box = "\n".join(text_content)
            ax.text(
                0.02,  # x position (left)
                0.98,  # y position (top)
                text_box,
                transform=ax.transAxes,
                color=color_text,
                verticalalignment="top",
                family="monospace",  # Use monospace font
                bbox=dict(
                    facecolor="white",
                    alpha=0.8,
                    edgecolor="none",
                    boxstyle="round,pad=0.5",
                ),
            )

        return fig

    return plotter
