import re
from datetime import datetime
from io import StringIO
import os
from pathlib import Path
import shlex
from typing import Union

import pandas as pd


# Regex to match the find -ls output line
_RE_LINE_PATTERN = re.compile(
    r"(?P<inode>\d+)\s+"
    r"(?P<blocks>\d+)\s+"
    r"(?P<permissions>[\w-]{10})\s+"
    r"(?P<hardlinks>\d+)\s+"
    r"(?P<owner>\S+)\s+"
    r"(?P<group>\S+)\s+"
    r"(?P<size>\d+)\s+"
    r"(?P<month>\w{3})\s+"
    r"(?P<day>\d{1,2})\s+"
    r"(?P<time_or_year>\d{2}:\d{2}|\d{4})\s+"
    r"(?P<filename>.+?)"
    r"(?:\s+->\s+(?P<symlink_target>.+))?$"
)


def _parse_line(line: str):
    # pylint: disable=W0718

    match = _RE_LINE_PATTERN.match(line.strip())
    if not match:
        return None
    parts = match.groupdict()

    # Parse datetime
    month = parts["month"]
    day = parts["day"]
    time_or_year = parts["time_or_year"]
    try:
        if ":" in time_or_year:
            # No year in output, use current year
            dt_str = f"{month} {day} {datetime.now().year} {time_or_year}"
            dt = datetime.strptime(dt_str, "%b %d %Y %H:%M")
        else:
            dt_str = f"{month} {day} {time_or_year}"
            dt = datetime.strptime(dt_str, "%b %d %Y")
    except Exception:
        dt = pd.NaT

    # Attempt shell split to get the full path
    try:
        full_path = shlex.split(parts["filename"].strip())[0]
    except ValueError:
        full_path = parts["filename"].strip()

    # Calculate directory and filename
    directory = os.path.dirname(full_path)
    filename = os.path.basename(full_path)

    return {
        "Inode": int(parts["inode"]),
        "Blocks": int(parts["blocks"]),
        "Permissions": parts["permissions"],
        "Hardlinks": int(parts["hardlinks"]),
        "Owner": parts["owner"],
        "Group": parts["group"],
        "Filesize": int(parts["size"]),
        "Datetime": dt,
        "Parent": directory,
        "Filename": filename,
        "SymlinkTarget": parts["symlink_target"],
        "IsSymlink": parts["permissions"].startswith("l"),
    }


def _parse_iterable(iterable):
    return pd.DataFrame(
        [parsed for line in iterable if (parsed := _parse_line(line))]
    )


def read_filetree(file: Union[str, Path, StringIO], common: bool = True) -> pd.DataFrame:
    """
    Reads the output of `find -ls` command and returns a DataFrame.
    Handles symlinks and combines Month, Day, Time/Year into a single Datetime column.
    """
    if isinstance(file, (str, Path)):
        with open(file, "r", encoding="utf-8") as f:
            df = _parse_iterable(f).sort_values(
                "Datetime", ignore_index=True
            )

    elif isinstance(file, StringIO):
        df = _parse_iterable(file).sort_values(
            "Datetime", ignore_index=True
        )
    else:
        raise TypeError(f"'file' must be str, Path, or StringIO, not {type(file).__name__}")

    if common:
        value = os.path.commonprefix(list(df["Filename"]))
        df["Filename"] = df["Filename"].apply(
            lambda x: x.removeprefix(value).removeprefix("/")
        )
    return df
