#!/bin/bash

set -e

# Parse arguments
while [[ "$#" -gt 0 ]]; do
    case $1 in
        -o|--output)
            output="$2"
            shift 2
            ;;
        -e|--ssh-command)
            ssh_cmd="$2"
            shift 2
            ;;
        -*)
            echo "Unknown option: $1"
            exit 1
            ;;
        *)
            if [[ -z "$expid" ]]; then
                expid="$1"
                shift
            else
                echo "Unexpected argument: $1"
                exit 1
            fi
            ;;
    esac
done

# Load .env file for default paths and credentials
env_file="$(dirname "$0")/../.env"
if [[ -f "$env_file" ]]; then
    set -a
    source "$env_file"
    set +a
fi

# Check if expid was provided
if [[ -z "$expid" ]]; then
    echo "Usage: $0 <expid> [--output <output_dir>] [--ssh-command <ssh_cmd>]"
    exit 1
fi

# Set default output if not provided
if [[ -z "$output" ]]; then
    if [[ -z "$logdir" ]]; then
        echo "Error: --output must be provided, or 'logdir' must be set in $env_file"
        exit 1
    fi
    output="$logdir/$expid"
fi

# Expand ~ and variables in output path
eval output="$output"

root="/appl/AS/AUTOSUBMIT_DATA/$expid/tmp/"

# Configure remote source and SSH command
if [[ -z "$ssh_cmd" ]]; then
    # Default behavior: use credentials from .env
    echo "Using credentials from .env file."
    required_vars=(file1 file2 user1 user2 machine1 machine2)
    for var in "${required_vars[@]}"; do
        if [[ -z "${!var}" ]]; then
            echo "Error: Required variable '$var' is not set in $env_file"
            exit 1
        fi
    done
    eval file1="$file1"
    eval file2="$file2"
    ssh_cmd="ssh -i $file2 -J $user1@$machine1 -i $file1"
    rsync_source_prefix="$user2@$machine2"
else
    # Custom SSH command provided. Assume it handles user/host.
    echo "Using custom SSH command."
    rsync_source_prefix=""
fi

# Create output subdirectories
mkdir -p "$output/ASFILES" "$output/LOGS" "$output/ASLOGS"

# Rsync commands
echo "Starting rsync..."
rsync -avPz -e "$ssh_cmd" --exclude '*/' ${rsync_source_prefix}:"$root" "$output/ASFILES"
rsync -avPz -e "$ssh_cmd" --exclude '*/' ${rsync_source_prefix}:"$root/LOG_$expid/" "$output/LOGS"
rsync -avPz -e "$ssh_cmd" --exclude '*/' ${rsync_source_prefix}:"$root/ASLOGS/" "$output/ASLOGS"

echo "Log transfer complete."
